@@ -97,7 +97,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc,v 1.26 2004/08/29 09:12:02 fyodor Exp $ */
+/* $Id: NmapOps.cc,v 1.28 2004/10/18 16:59:36 fyodor Exp $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -200,6 +200,8 @@ void NmapOps::Initialize() {
   initial_rtt_timeout = INITIAL_RTT_TIMEOUT;
   min_host_group_sz = 1;
   max_host_group_sz = 100000; // don't want to be restrictive unles user sets
+  max_tcp_scan_delay = MAX_TCP_SCAN_DELAY;
+  max_udp_scan_delay = MAX_UDP_SCAN_DELAY;
   max_ips_to_scan = 0;
   extra_payload_length = 0;
   extra_payload = NULL;
@@ -300,7 +302,7 @@ void NmapOps::ValidateOptions() {
  }
 
  if (ipprotscan + (TCPScan() || UDPScan()) + listscan + pingscan > 1) {
-   fatal("Sorry, the IPProtoscan, Listscan, and Pingscan (-sO, -sL, -sP) must currently be used alone rathre than combined with other scan types.");
+   fatal("Sorry, the IPProtoscan, Listscan, and Pingscan (-sO, -sL, -sP) must currently be used alone rather than combined with other scan types.");
  }
 
  if ((pingscan && pingtype == PINGTYPE_NONE)) {
