@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.h,v 1.106 2004/08/29 09:12:03 fyodor Exp $ */
+/* $Id: nmap.h,v 1.107 2004/10/18 16:59:37 fyodor Exp $ */
 
 #ifndef NMAP_H
 #define NMAP_H
@@ -307,6 +307,15 @@ void *realloc();
 				 trip */
 #endif
 
+/* Default maximum send delay between probes to the same host */
+#ifndef MAX_TCP_SCAN_DELAY
+#define MAX_TCP_SCAN_DELAY 1000
+#endif
+
+#ifndef MAX_UDP_SCAN_DELAY
+#define MAX_UDP_SCAN_DELAY 1000
+#endif
+
 /* We wait at least 100 ms for a response by default - while that
    seems aggressive, waiting too long can cause us to fail to detect
    drops until many probes later on extremely low-latency
